#include <glm/mat4x4.hpp>
#include <glm/mat3x3.hpp>
#include <glm/ext.hpp>
#include "skybox.h"
#include "shader.h"
#include "texture.h"
#include "camera.h"
#include "opengl.h"
#include <resource.h>

#define BUFFER_OFFSET(i) ((char *)NULL + (i))

RESOURCE_PACK(shaders)

Skybox::Skybox(Texture* myCubeMap)
{
    cubeMap = myCubeMap;

    glGenVertexArrays(1, &vao);
    glBindVertexArray(vao);
    glGenBuffers(2, vbos);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, vbos[0]);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, 36 * sizeof(GLubyte), skyboxIndices, GL_STATIC_DRAW);
    
	glBindBuffer(GL_ARRAY_BUFFER, vbos[1]);
    glBufferData(GL_ARRAY_BUFFER, 24 * sizeof(GLfloat), skyboxVertices, GL_STATIC_DRAW);
    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3*sizeof(GLfloat), BUFFER_OFFSET(0));
    glEnableVertexAttribArray(0);
	
    Resource::ResourceMap shaderMap;
    Resource::getResourcePack_shaders(shaderMap);
    std::string vertSource = shaderMap["shaders/skybox.vert.glsl"];
    std::string fragSource = shaderMap["shaders/skybox.frag.glsl"];
    shader = new Shader(vertSource, fragSource);
    mvpLocation = shader->getLocation("MVP");
    cubemapLocation = shader->getLocation("skybox");

    glBindVertexArray(0);
}

Skybox::~Skybox()
{
    glDeleteVertexArrays(1, &vao);
    glDeleteBuffers(2, vbos);
}

void Skybox::renderGL(Camera* myCamera)
{
    glm::mat4 viewMatrix = glm::mat4(glm::mat3(myCamera->getViewMatrix()));
	
    shader->bind();
    shader->bindMat4(mvpLocation, myCamera->getProjectionMatrix() * viewMatrix);
    shader->bindInteger(cubemapLocation, 0);
    cubeMap->bind(0);
    glBindVertexArray(vao);

    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, vbos[0]);
    glDrawElements(GL_TRIANGLES, 36, GL_UNSIGNED_BYTE, NULL);
    glBindVertexArray(0);
}

const GLfloat Skybox::skyboxVertices[] = {
    -1.0f,  1.0f,  1.0f,
    -1.0f,  1.0f, -1.0f,
    -1.0f, -1.0f,  1.0f,
    -1.0f, -1.0f, -1.0f,
     1.0f,  1.0f,  1.0f,
     1.0f,  1.0f, -1.0f,
     1.0f, -1.0f,  1.0f,
     1.0f, -1.0f, -1.0f
};

const GLubyte Skybox::skyboxIndices[] = {
    0, 6, 2,
    0, 4, 6,
    1, 3, 5,
    3, 7, 5,
    0, 3, 1,
    0, 2, 3,
    4, 5, 6,
    5, 7, 6,
    0, 5, 4,
    0, 1, 5,
    2, 6, 3,
    6, 7, 3
};
